@@ -127,6 +127,10 @@ gst_type_find_element_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&type_find_element_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&type_find_element_sink_template));
   gst_element_class_set_details (gstelement_class,
       &gst_type_find_element_details);
 }
@@ -426,9 +430,13 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
               GST_PLUGIN_FEATURE_NAME (entry->factory), entry->probability);
           g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE], 0,
               entry->probability, entry->caps);
+          stop_typefinding (typefind);
+          gst_pad_event_default (pad, event);
+        } else {
+          gst_pad_event_default (pad, event);
+          GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+          stop_typefinding (typefind);
         }
-        stop_typefinding (typefind);
-        gst_pad_event_default (pad, event);
         break;
       default:
         gst_data_unref (GST_DATA (event));
@@ -609,7 +617,9 @@ gst_type_find_element_chain (GstPad * pad, GstData * data)
       if (typefind->caps) {
         stop_typefinding (typefind);
       } else if (typefind->possibilities == NULL) {
+      error:
         GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+        stop_typefinding (typefind);
       } else {
         /* set up typefind element for next iteration */
         typefind->possibilities =
@@ -624,6 +634,7 @@ gst_type_find_element_chain (GstPad * pad, GstData * data)
         }
         if (!walk) {
           /* find out if we should seek */
+        restart:
           for (walk = typefind->possibilities; walk; walk = g_list_next (walk)) {
             entry = (TypeFindEntry *) walk->data;
             if (entry->requested_size > 0) {
@@ -652,8 +663,19 @@ gst_type_find_element_chain (GstPad * pad, GstData * data)
                 GST_DEBUG_OBJECT (typefind,
                     "'%s' was reset - couldn't seek to %" G_GINT64_FORMAT,
                     GST_PLUGIN_FEATURE_NAME (entry->factory), seek_offset);
-                entry->requested_size = 0;
-                entry->requested_offset = 0;
+                if (entry->probability == 0) {
+                  free_entry (entry);
+                  typefind->possibilities =
+                      g_list_delete_link (typefind->possibilities, walk);
+                  /* FIXME: too many gotos */
+                  if (!typefind->possibilities)
+                    goto error;
+                  /* we modified the list, let's restart */
+                  goto restart;
+                } else {
+                  entry->requested_size = 0;
+                  entry->requested_offset = 0;
+                }
               }
             }
           }
