@@ -1294,8 +1294,11 @@ gst_caps_do_simplify (GstCaps * caps)
 xmlNodePtr
 gst_caps_save_thyself (const GstCaps * caps, xmlNodePtr parent)
 {
+  char *s = gst_caps_to_string (caps);
 
-  return 0;
+  xmlNewChild (parent, NULL, "caps", s);
+  g_free (s);
+  return parent;
 }
 
 /**
@@ -1309,6 +1312,9 @@ gst_caps_save_thyself (const GstCaps * caps, xmlNodePtr parent)
 GstCaps *
 gst_caps_load_thyself (xmlNodePtr parent)
 {
+  if (strcmp ("caps", parent->name) == 0) {
+    return gst_caps_from_string (xmlNodeGetContent (parent));
+  }
 
   return NULL;
 }
