@@ -198,16 +198,9 @@ gst_scheduler_pad_unlink (GstScheduler * sched, GstPad * srcpad,
 GstPad *
 gst_scheduler_pad_select (GstScheduler * sched, GList * padlist)
 {
-  GstSchedulerClass *sclass;
-
   g_return_val_if_fail (GST_IS_SCHEDULER (sched), NULL);
   g_return_val_if_fail (padlist != NULL, NULL);
 
-  sclass = GST_SCHEDULER_GET_CLASS (sched);
-
-  if (sclass->pad_select)
-    sclass->pad_select (sched, padlist);
-
   return NULL;
 }
 
@@ -418,15 +411,8 @@ gst_scheduler_remove_scheduler (GstScheduler * sched, GstScheduler * sched2)
 void
 gst_scheduler_lock_element (GstScheduler * sched, GstElement * element)
 {
-  GstSchedulerClass *sclass;
-
   g_return_if_fail (GST_IS_SCHEDULER (sched));
   g_return_if_fail (GST_IS_ELEMENT (element));
-
-  sclass = GST_SCHEDULER_GET_CLASS (sched);
-
-  if (sclass->lock_element)
-    sclass->lock_element (sched, element);
 }
 
 /**
@@ -445,9 +431,6 @@ gst_scheduler_unlock_element (GstScheduler * sched, GstElement * element)
   g_return_if_fail (GST_IS_ELEMENT (element));
 
   sclass = GST_SCHEDULER_GET_CLASS (sched);
-
-  if (sclass->unlock_element)
-    sclass->unlock_element (sched, element);
 }
 
 /**
@@ -527,7 +510,7 @@ gst_scheduler_interrupt (GstScheduler * sched, GstElement * element)
  * gst_scheduler_get_clock:
  * @sched: the scheduler
  *
- * Get the current clock used by the scheduler
+ * Gets the current clock used by the scheduler.
  *
  * Returns: a GstClock
  */
@@ -540,8 +523,8 @@ gst_scheduler_get_clock (GstScheduler * sched)
   if (GST_FLAG_IS_SET (sched, GST_SCHEDULER_FLAG_FIXED_CLOCK)) {
     clock = sched->clock;
 
-    GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler using fixed clock %p (%s)", clock,
-        (clock ? GST_OBJECT_NAME (clock) : "nil"));
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler using fixed clock %p (%s)",
+        clock, clock ? GST_STR_NULL (GST_OBJECT_NAME (clock)) : "-");
   } else {
     GList *schedulers = sched->schedulers;
     GList *providers = sched->clock_providers;
@@ -551,24 +534,34 @@ gst_scheduler_get_clock (GstScheduler * sched)
       GstScheduler *scheduler = GST_SCHEDULER (schedulers->data);
 
       clock = gst_scheduler_get_clock (scheduler);
-      if (clock)
+      if (clock) {
+        GST_CAT_DEBUG (GST_CAT_CLOCK,
+            "scheduler found managed sched clock %p (%s)",
+            clock, clock ? GST_STR_NULL (GST_OBJECT_NAME (clock)) : "-");
         break;
+      }
 
       schedulers = g_list_next (schedulers);
     }
     /* still no clock, try to find one in the providers */
     while (!clock && providers) {
       clock = gst_element_get_clock (GST_ELEMENT (providers->data));
-
+      if (clock)
+        GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler found provider clock: %p (%s)",
+            clock, clock ? GST_STR_NULL (GST_OBJECT_NAME (clock)) : "-");
       providers = g_list_next (providers);
     }
     /* still no clock, use a system clock */
     if (!clock && sched->parent_sched == NULL) {
       clock = gst_system_clock_obtain ();
+      /* we unref since this function is not supposed to increase refcount
+       * of clock object returned; this is ok since the systemclock always
+       * has a refcount of at least one in the current code. */
+      gst_object_unref (GST_OBJECT (clock));
+      GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler obtained system clock: %p (%s)",
+          clock, clock ? GST_STR_NULL (GST_OBJECT_NAME (clock)) : "-");
     }
   }
-  GST_CAT_LOG_OBJECT (GST_CAT_CLOCK, sched, "scheduler selected clock %p (%s)",
-      clock, clock ? GST_STR_NULL (GST_OBJECT_NAME (clock)) : "-");
 
   return clock;
 }
@@ -601,8 +594,8 @@ gst_scheduler_use_clock (GstScheduler * sched, GstClock * clock)
  * @sched: the scheduler
  * @clock: the clock to set
  *
- * Set the clock for the scheduler. The clock will be distributed 
- * to all the elements managed by the scheduler. 
+ * Set the clock for the scheduler. The clock will be distributed
+ * to all the elements managed by the scheduler.
  */
 void
 gst_scheduler_set_clock (GstScheduler * sched, GstClock * clock)
@@ -764,6 +757,7 @@ gst_scheduler_factory_get_type (void)
       (GInstanceInitFunc) gst_scheduler_factory_init,
       NULL
     };
+
     schedulerfactory_type = g_type_register_static (GST_TYPE_PLUGIN_FEATURE,
         "GstSchedulerFactory", &schedulerfactory_info, 0);
   }
@@ -799,6 +793,45 @@ gst_scheduler_factory_init (GstSchedulerFactory * factory)
 }
 
 
+/**
+ * gst_scheduler_register:
+ * @plugin: a #GstPlugin
+ * @name: name of the scheduler to register
+ * @longdesc: description of the scheduler
+ * @type: #GType of the scheduler to register
+ *
+ * Registers a scheduler with GStreamer.
+ *
+ * Returns: TRUE, if the registering succeeded, FALSE on error.
+ *
+ * Since: 0.8.5
+ **/
+gboolean
+gst_scheduler_register (GstPlugin * plugin, const gchar * name,
+    const gchar * longdesc, GType type)
+{
+  GstSchedulerFactory *factory;
+
+  g_return_val_if_fail (plugin != NULL, FALSE);
+  g_return_val_if_fail (name != NULL, FALSE);
+  g_return_val_if_fail (longdesc != NULL, FALSE);
+  g_return_val_if_fail (g_type_is_a (type, GST_TYPE_SCHEDULER), FALSE);
+
+  factory = gst_scheduler_factory_find (name);
+  if (factory) {
+    g_return_val_if_fail (factory->type == 0, FALSE);
+    g_free (factory->longdesc);
+    factory->longdesc = g_strdup (longdesc);
+    factory->type = type;
+  } else {
+    factory = gst_scheduler_factory_new (name, longdesc, type);
+    g_return_val_if_fail (factory, FALSE);
+    gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  }
+
+  return TRUE;
+}
+
 /**
  * gst_scheduler_factory_new:
  * @name: name of schedulerfactory to create
