@@ -223,6 +223,8 @@ gst_thread_dispose (GObject * object)
 
   g_assert (GST_STATE (thread) == GST_STATE_NULL);
 
+  GST_CAT_DEBUG (GST_CAT_REFCOUNTING, "GstThread: dispose, freeing locks");
+
   g_mutex_free (thread->lock);
   g_cond_free (thread->cond);
 
@@ -369,12 +371,14 @@ gst_thread_catch (GstThread * thread)
   if (thread == gst_thread_get_current ()) {
     /* we're trying to catch ourself */
     if (!GST_FLAG_IS_SET (thread, GST_THREAD_MUTEX_LOCKED)) {
+      GST_DEBUG_OBJECT (thread, "catching itself, grabbing lock");
       g_mutex_lock (thread->lock);
       GST_FLAG_SET (thread, GST_THREAD_MUTEX_LOCKED);
     }
     GST_DEBUG_OBJECT (thread, "catching itself");
     GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
   } else {
+    GST_DEBUG_OBJECT (thread, "catching thread, grabbing lock");
     /* another thread is trying to catch us */
     g_mutex_lock (thread->lock);
     wait = !GST_FLAG_IS_SET (thread, GST_THREAD_STATE_SPINNING);
@@ -478,37 +482,39 @@ gst_thread_change_state (GstElement * element)
          by ourself (ouch) */
       GST_LOG_OBJECT (thread, "destroying GThread %p", thread->thread_id);
       GST_FLAG_SET (thread, GST_THREAD_STATE_REAPING);
-      thread->thread_id = NULL;
-      if (thread == gst_thread_get_current ()) {
-        /* or should we continue? */
-        g_warning
-            ("Thread %s is destroying itself. Function call will not return!",
-            GST_ELEMENT_NAME (thread));
-        gst_scheduler_reset (GST_ELEMENT_SCHED (thread));
+      /* thread was already gone */
+      if (thread->thread_id != NULL) {
+        thread->thread_id = NULL;
+        if (thread == gst_thread_get_current ()) {
+          /* or should we continue? */
+          GST_LOG_OBJECT (thread,
+              "Thread %s is destroying itself. Function call will not return!",
+              GST_ELEMENT_NAME (thread));
+          gst_scheduler_reset (GST_ELEMENT_SCHED (thread));
 
-        /* unlock and signal - we are out */
-        gst_thread_release (thread);
+          /* unlock and signal - we are out, note that gst_thread_release does
+           * nothing when we are running in the thread context */
+          GST_DEBUG_OBJECT (thread, "releasing lock");
+          g_cond_signal (thread->cond);
+          g_mutex_unlock (thread->lock);
 
-        GST_INFO_OBJECT (thread, "GThread %p is exiting", g_thread_self ());
+          GST_INFO_OBJECT (thread, "GThread %p is exiting", g_thread_self ());
 
-        g_signal_emit (G_OBJECT (thread), gst_thread_signals[SHUTDOWN], 0);
+          g_signal_emit (G_OBJECT (thread), gst_thread_signals[SHUTDOWN], 0);
 
-        g_thread_exit (NULL);
+          g_thread_exit (NULL);
+          return GST_STATE_SUCCESS;
+        }
+        /* now wait for the thread to destroy itself */
+        GST_DEBUG_OBJECT (thread, "signal");
+        g_cond_signal (thread->cond);
+        GST_DEBUG_OBJECT (thread, "wait");
+        g_cond_wait (thread->cond, thread->lock);
+        GST_DEBUG_OBJECT (thread, "done");
+        /* it should be dead now */
       }
-      /* now wait for the thread to destroy itself */
-      g_cond_signal (thread->cond);
-      g_cond_wait (thread->cond, thread->lock);
-      /* it should be dead now */
       break;
     default:
-      GST_ERROR_OBJECT (element, "unhandled state change! %x",
-          GST_STATE_TRANSITION (element));
-      g_warning ("thread %s: UNHANDLED STATE CHANGE! %x",
-          GST_STR_NULL (GST_OBJECT_NAME (element)),
-          GST_STATE_TRANSITION (element));
-      /* FIXME: not doable with current threading mess:
-         g_assert_not_reached ();
-       */
       break;
   }
 
@@ -544,7 +550,7 @@ gst_thread_child_state_change (GstBin * bin, GstElementState oldstate,
       gst_element_state_get_name (newstate));
   if (parent_class->child_state_change)
     parent_class->child_state_change (bin, oldstate, newstate, element);
-  /* We'll wake up the main thread now. Note that we can't lock the thread here, 
+  /* We'll wake up the main thread now. Note that we can't lock the thread here,
      because we might be called from inside gst_thread_change_state when holding
      the lock. But this doesn't cause any problems. */
   if (newstate == GST_STATE_PLAYING)
@@ -563,6 +569,7 @@ gst_thread_main_loop (void *arg)
 {
   GstThread *thread = NULL;
   gboolean status;
+  GstScheduler *sched;
 
   thread = GST_THREAD (arg);
   g_mutex_lock (thread->lock);
@@ -584,6 +591,8 @@ gst_thread_main_loop (void *arg)
       while (status && GST_FLAG_IS_SET (thread, GST_THREAD_STATE_SPINNING)) {
         g_mutex_unlock (thread->lock);
         status = gst_bin_iterate (GST_BIN (thread));
+        if (!status)
+          GST_DEBUG_OBJECT (thread, "iterate returned false");
         if (GST_FLAG_IS_SET (thread, GST_THREAD_MUTEX_LOCKED)) {
           GST_FLAG_UNSET (thread, GST_THREAD_MUTEX_LOCKED);
         } else {
@@ -601,7 +610,9 @@ gst_thread_main_loop (void *arg)
 
   /* we need to destroy the scheduler here because it has mapped it's
    * stack into the threads stack space */
-  gst_scheduler_reset (GST_ELEMENT_SCHED (thread));
+  sched = GST_ELEMENT_SCHED (thread);
+  if (sched)
+    gst_scheduler_reset (sched);
 
   /* must do that before releasing the lock - we might get disposed before being done */
   g_signal_emit (G_OBJECT (thread), gst_thread_signals[SHUTDOWN], 0);
