@@ -447,25 +447,20 @@ gst_opt_scheduler_dispose (GObject * object)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GstSchedulerFactory *factory;
+#ifdef USE_COTHREADS
+  if (!gst_scheduler_register (plugin, "opt" COTHREADS_NAME,
+          "An optimal scheduler using " COTHREADS_NAME " cothreads",
+          gst_opt_scheduler_get_type ()))
+#else
+  if (!gst_scheduler_register (plugin, "opt",
+          "An optimal scheduler using no cothreads",
+          gst_opt_scheduler_get_type ()))
+#endif
+    return FALSE;
 
   GST_DEBUG_CATEGORY_INIT (debug_scheduler, "scheduler", 0,
       "optimal scheduler");
 
-#ifdef USE_COTHREADS
-  factory = gst_scheduler_factory_new ("opt" COTHREADS_NAME,
-      "An optimal scheduler using " COTHREADS_NAME " cothreads",
-      gst_opt_scheduler_get_type ());
-#else
-  factory = gst_scheduler_factory_new ("opt",
-      "An optimal scheduler using no cothreads", gst_opt_scheduler_get_type ());
-#endif
-
-  if (factory != NULL) {
-    gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  } else {
-    g_warning ("could not register scheduler: optimal");
-  }
   return TRUE;
 }
 
@@ -915,6 +910,10 @@ remove_from_group (GstOptSchedulerGroup * group, GstElement * element)
    * this element */
   group_dec_links_for_element (group, element);
 
+  if (gst_element_get_state (element) == GST_STATE_PLAYING) {
+    group_element_set_enabled (group, element, FALSE);
+  }
+
   group->elements = g_slist_remove (group->elements, element);
   group->num_elements--;
 
@@ -966,7 +965,7 @@ group_num_elements (GstOptSchedulerGroup * group)
  * as decoupled elements are added as entry but are not added to the elements 
  * list */
 static gboolean
-group_has_element (GstOptSchedulerGroup * group, GstElement *element)
+group_has_element (GstOptSchedulerGroup * group, GstElement * element)
 {
   if (group->entry == element)
     return TRUE;
@@ -1213,7 +1212,7 @@ gst_opt_scheduler_schedule_run_queue (GstOptScheduler * osched)
       res = schedule_group (group);
     } else {
       GST_INFO_OBJECT (osched,
-           "group was disabled while it was on the queue, not scheduling");
+          "group was disabled while it was on the queue, not scheduling");
       res = TRUE;
     }
     if (!res) {
@@ -1549,13 +1548,15 @@ gst_opt_scheduler_state_transition (GstScheduler * sched, GstElement * element,
   GstOptSchedulerGroup *group;
   GstElementStateReturn res = GST_STATE_SUCCESS;
 
-  GST_DEBUG ("element \"%s\" state change (%04x)", GST_ELEMENT_NAME (element),
+  GST_DEBUG ("element \"%s\" state change (%04x)",
+      GST_ELEMENT_NAME (element) ? GST_ELEMENT_NAME (element) : "(null)",
       transition);
 
   /* we check the state of the managing pipeline here */
   if (GST_IS_BIN (element)) {
     if (GST_SCHEDULER_PARENT (sched) == element) {
-      GST_LOG ("parent \"%s\" changed state", GST_ELEMENT_NAME (element));
+      GST_LOG ("parent \"%s\" changed state",
+          GST_ELEMENT_NAME (element) ? GST_ELEMENT_NAME (element) : "(null)");
 
       switch (transition) {
         case GST_STATE_PLAYING_TO_PAUSED:
@@ -2117,7 +2118,7 @@ gst_opt_scheduler_pad_link (GstScheduler * sched, GstPad * srcpad,
       if (!group1) {
         /* create a new group for src_element as it cannot be merged into another group
          * here. we create the group in the same chain as the loop-based element. note
-         * that creating a new group will also increment the links with other groups */
+         * that creating a new group will not increment the links with other groups */
         GST_DEBUG ("creating new group for element %s",
             GST_ELEMENT_NAME (src_element));
         group1 =
@@ -2127,9 +2128,9 @@ gst_opt_scheduler_pad_link (GstScheduler * sched, GstPad * srcpad,
         /* both elements are already in a group, make sure they are added to
          * the same chain */
         merge_chains (group1->chain, group2->chain);
-        /* increment the group link counters */
-        group_inc_link (group1, group2);
       }
+      /* increment the group link counters */
+      group_inc_link (group1, group2);
       break;
     }
     case GST_OPT_INVALID:
@@ -2440,7 +2441,8 @@ group_migrate_connected (GstOptScheduler * osched, GstElement * element,
         new_group->type != GST_OPT_SCHEDULER_GROUP_LOOP) {
       GST_LOG ("removing last element from new group");
       new_group =
-          remove_from_group (new_group, GST_ELEMENT (new_group->elements->data));
+          remove_from_group (new_group,
+          GST_ELEMENT (new_group->elements->data));
       return NULL;
     }
     /* at this point the new group lives in its own chain but might
