@@ -10,13 +10,13 @@
 gboolean running = FALSE;
 
 static void
-construct_pipeline (GstElement *pipeline) 
+construct_pipeline (GstElement * pipeline)
 {
   GstElement *src, *sink, *identity;
 
-  src      = gst_element_factory_make ("fakesrc",  NULL);
+  src = gst_element_factory_make ("fakesrc", NULL);
   identity = gst_element_factory_make ("identity", NULL);
-  sink     = gst_element_factory_make ("fakesink", NULL);
+  sink = gst_element_factory_make ("fakesink", NULL);
   g_assert (src);
   g_assert (identity);
   g_assert (sink);
@@ -29,43 +29,45 @@ construct_pipeline (GstElement *pipeline)
 }
 
 void
-state_changed (GstElement *el, gint arg1, gint arg2, gpointer user_data)
+state_changed (GstElement * el, gint arg1, gint arg2, gpointer user_data)
 {
   GstElementState state = gst_element_get_state (el);
-  
-  g_print ("element %s has changed state to %s\n", 
-	   GST_ELEMENT_NAME (el), 
-	   gst_element_state_get_name (state));
-  if (state == GST_STATE_PLAYING) running = TRUE;
+
+  g_print ("element %s has changed state to %s\n",
+      GST_ELEMENT_NAME (el), gst_element_state_get_name (state));
+  if (state == GST_STATE_PLAYING)
+    running = TRUE;
   /* if we move from PLAYING to PAUSED, we're done */
-  if (state == GST_STATE_PAUSED && running) gst_main_quit ();
+  if (state == GST_STATE_PAUSED && running) {
+    running = FALSE;
+    gst_main_quit ();
+  }
 }
 
 int
-main (gint argc, gchar *argv[])
+main (gint argc, gchar * argv[])
 {
   int runs = 100;
   int i;
   gulong id;
   GstElement *thread;
-  
+
   gst_init (&argc, &argv);
 
-  for (i = 0; i < runs; ++i)
-  {
+  for (i = 0; i < runs; ++i) {
     thread = gst_thread_new ("main_thread");
     g_assert (thread);
 
     /* connect state change signal */
-    id = g_signal_connect (G_OBJECT (thread), "state_change", 
-		           G_CALLBACK (state_changed), NULL);
+    id = g_signal_connect (G_OBJECT (thread), "state_change",
+        G_CALLBACK (state_changed), NULL);
     construct_pipeline (thread);
 
     g_print ("Setting thread to play\n");
     gst_element_set_state (thread, GST_STATE_PLAYING);
 
     g_print ("Going into the main GStreamer loop\n");
-    gst_main (); 
+    gst_main ();
     g_print ("Coming out of the main GStreamer loop\n");
     g_signal_handler_disconnect (G_OBJECT (thread), id);
     gst_element_set_state (thread, GST_STATE_NULL);
@@ -76,4 +78,3 @@ main (gint argc, gchar *argv[])
 
   return 0;
 }
-
