@@ -66,6 +66,7 @@ static guint64 max = 0;
 static GstClock *s_clock;
 static GstElement *pipeline;
 gboolean caught_intr = FALSE;
+gboolean caught_error = FALSE;
 
 gboolean
 idle_func (gpointer data)
@@ -85,13 +86,32 @@ idle_func (gpointer data)
   min = MIN (min, diff);
   max = MAX (max, diff);
 
-  if (!busy || caught_intr || (max_iterations > 0
-          && iterations >= max_iterations)) {
+  if (!busy || caught_intr || caught_error ||
+      (max_iterations > 0 && iterations >= max_iterations)) {
+    char *s_iterations;
+    char *s_sum;
+    char *s_ave;
+    char *s_min;
+    char *s_max;
+
     gst_main_quit ();
-    g_print (_("Execution ended after %" G_GUINT64_FORMAT " iterations (sum %"
-            G_GUINT64_FORMAT " ns, average %" G_GUINT64_FORMAT " ns, min %"
-            G_GUINT64_FORMAT " ns, max %" G_GUINT64_FORMAT " ns).\n"),
-        iterations, sum, sum / iterations, min, max);
+
+    /* We write these all to strings first because 
+     * G_GUINT64_FORMAT and gettext mix very poorly */
+    s_iterations = g_strdup_printf ("%" G_GUINT64_FORMAT, iterations);
+    s_sum = g_strdup_printf ("%" G_GUINT64_FORMAT, sum);
+    s_ave = g_strdup_printf ("%" G_GUINT64_FORMAT, sum / iterations);
+    s_min = g_strdup_printf ("%" G_GUINT64_FORMAT, min);
+    s_max = g_strdup_printf ("%" G_GUINT64_FORMAT, max);
+
+    g_print (_("Execution ended after %s iterations (sum %s ns, "
+            "average %s ns, min %s ns, max %s ns).\n"),
+        s_iterations, s_sum, s_ave, s_min, s_max);
+    g_free (s_iterations);
+    g_free (s_sum);
+    g_free (s_ave);
+    g_free (s_min);
+    g_free (s_max);
   }
 
   return busy;
@@ -273,7 +293,8 @@ print_tag (const GstTagList * list, const gchar * tag, gpointer unused)
     gchar *str;
 
     if (gst_tag_get_type (tag) == G_TYPE_STRING) {
-      g_assert (gst_tag_list_get_string_index (list, tag, i, &str));
+      if (!gst_tag_list_get_string_index (list, tag, i, &str))
+        g_assert_not_reached ();
     } else {
       str =
           g_strdup_value_contents (gst_tag_list_get_value_index (list, tag, i));
@@ -288,6 +309,7 @@ print_tag (const GstTagList * list, const gchar * tag, gpointer unused)
     g_free (str);
   }
 }
+
 static void
 found_tag (GObject * pipeline, GstElement * source, GstTagList * tags)
 {
@@ -296,6 +318,13 @@ found_tag (GObject * pipeline, GstElement * source, GstTagList * tags)
   gst_tag_list_foreach (tags, print_tag, NULL);
 }
 
+static void
+error_cb (GObject * object, GstObject * source, GError * error, gchar * debug)
+{
+  gst_element_default_error (object, source, error, debug);
+  caught_error = TRUE;
+}
+
 #ifndef DISABLE_FAULT_HANDLER
 /* we only use sighandler here because the registers are not important */
 static void
@@ -461,7 +490,7 @@ main (int argc, char *argv[])
     } else {
       fprintf (stderr, _("ERROR: pipeline could not be constructed.\n"));
     }
-    exit (1);
+    return 1;
   } else if (error) {
     fprintf (stderr, _("WARNING: erroneous pipeline: %s\n"), error->message);
     fprintf (stderr, _("         Trying to run anyway.\n"));
@@ -477,8 +506,7 @@ main (int argc, char *argv[])
   if (tags) {
     g_signal_connect (pipeline, "found-tag", G_CALLBACK (found_tag), NULL);
   }
-  g_signal_connect (pipeline, "error", G_CALLBACK (gst_element_default_error),
-      NULL);
+  g_signal_connect (pipeline, "error", G_CALLBACK (error_cb), NULL);
 
 #ifndef GST_DISABLE_LOADSAVE
   if (savefile) {
@@ -493,7 +521,7 @@ main (int argc, char *argv[])
 
       if (real_pipeline == NULL) {
         fprintf (stderr, _("ERROR: the 'pipeline' element wasn't found.\n"));
-        exit (1);
+        return 1;
       }
       gst_bin_add (GST_BIN (real_pipeline), pipeline);
       pipeline = real_pipeline;
@@ -517,6 +545,10 @@ main (int argc, char *argv[])
       gst_element_wait_state_change (pipeline);
       g_print ("got the state change.\n");
     }
+    if (caught_intr)
+      res = 2;
+    if (caught_error)
+      res = 3;
 
     gst_element_set_state (pipeline, GST_STATE_NULL);
   }
